#
# inetclient.rb version 1.3.1
#
#     Copyright(c) 1999 Minero Aoki
#     aamine@dp.u-netsurf.ne.jp
#

require 'net/session'

require 'extmod'
require 'must'
require 'bug'
require 'dispatch'



class INETclient

  Version = '1.3.1'

  def login( *args )
    return if logged_in?
    @session.start( INETsocket, *args )
    do_login
  end

  def do_login()  end ; private :do_login
  def do_logout() end ; private :do_logout

  def logout
    return unless logged_in?
    do_logout
    @session.finish
  end

  def logged_in?
    @session.active?
  end

  class << self
    def login( addr, port, *args )
      cl = new( addr, port )
      cl.login( *args )
      if iterator?
        yield cl
        cl.logout
      end
    end
  end

  def address
    @session.address
  end

  def port
    @session.port
  end

  def socket
    @session.socket
  end

  def proto_type
    @session.proto_type
  end

  def proto
    @session.proto
  end

  def proto=( arg )
    @session.proto = arg
  end

end

INETClient = INETclient


class INETsocket < Net::ProtocolSocket

  include Dispatchable

  def initialize( addr, port, pipe = nil )
    addr.must String
    port.must Integer
    super( addr, port, pipe )

    @pipe << "TCP connection to #{addr} opened" if @pipe
  end

  property :socket, TCPsocket, true, true

  event_handler :on_writing
  event_handler :on_write
  event_handler :on_writeall

  event_handler :on_reading
  event_handler :on_read
  event_handler :on_readall


  def writeline( str )
    str.must String
    super
  end


  private

  def do_write_beg
    super
    dispatch :on_writing
  end

  def do_write_do( arg )
    ret = super
    dispatch :on_write, ret
    return ret
  end

  def do_write_fin
    ret = super
    dispatch :on_writeall, ret
    return ret
  end


  public

  def read( len, &blk )
    do_read(blk){ super( len ) }
  end

  def readuntil( targ, &blk )
    do_read(blk){ super( targ ) }
  end
      
  def read_pendstr( ret = '', &blk )
    do_read(blk){ super( ret ) }
  end

  def read_pendlist
    do_read(nil){ super }
  end


  private

  def do_read( block )
    @rlen = 0
    push_event( :on_read, block ) if block
    dispatch :on_reading
    ret = yield
    dispatch :on_readall
    pop_event( :on_read ) if block

    return ret
  end

  def fill_rbuf
    tmp = @socket.sysread( READ_BLOCK )
    @buffer << tmp
                                  #$stderr.puts ">>>sysread size=#{tmp.size}"
    dispatch :on_read, tmp.size
  end

end
